import os

#-------------------CFG FILE--------------#
# change just the json, geo and location of the results
def write_cfg(N,path,Dim):
    title=str(N)+'_sphere.cfg'

    template = """directory=toolboxes/fluid/moving_body/{N}_sphere/ 
case.dimension={Dim}
[fluid]
filename=$cfgdir/{N}_sphere.json 
mesh.filename=$cfgdir/{N}_sphere.geo 
gmsh.hsize=3
#solver=Oseen #Oseen,Picard,Newton
ksp-monitor=true
pc-type=lu
ksp-type=preonly
#reuse-prec=1
ksp-maxit-reuse=20
snes-monitor=true
snes-maxit=100
define-pressure-cst=true
#define-pressure-cst.method=lagrange-multiplier#algebraic
verbose_solvertimer=1
#body.articulation.method=p-matrix
[fluid.alemesh]
pc-type=lu
[fluid.bdf]
order=2
[ts]
time-step=0.1#23
time-final=1.0
#restart=true
restart.at-last-save=true
time-initial=-0.1#0#-0.1
#save.freq=2"""
    context = {"N":N,"Dim":Dim}
    with  open(os.path.join(path,title),'w') as myfile:
        myfile.write(template.format(**context))

#-------------------JSON FILE--------------#
# repeats the material section and body section for each new sphere
# the articulations are attached to the sphere on the right of the shrinking link
# !!! for the moment, the bounday conditions have not been tuned !!!
# indices are from 0 to N-1

def write_json(N,action,path,Dim,RelativeVelocity):
    Relative_Vel = ["0*pulse(t, 0, 1-eps, 1):t:eps" for i in range(0, N-1)]
    if action[0:3] == 'ext':
        Relative_Vel[int(action[10])] = "-4*pulse(t, 0, 1-eps, 1):t:eps"
    elif action[0:3] == 'ret':
        Relative_Vel[int(action[11])] = "4*pulse(t, 0, 1-eps, 1):t:eps"

    if Dim==2:   
        title=str(N)+'_sphere.json'
        template="""{{
        "Name": "{N}_sphere swimmer",
        "ShortName":"{N}_sphere swimmer",
        "Models":
        {{
            "equations":"Stokes"
        }},
        "Materials":
        {{
            "Fluid":{{
                "physics":"fluid",
                "rho":"1",
                "mu":"1"
            }}{Material_Block_Nsphere}
        }},
        "Parameters":
        {{
            "eps":1e-10
        }},
        "BoundaryConditions":
        {{
            "velocity":
            {{
                "Dirichlet":
                {{
                    "BoxWalls":
                    {{
                        "expr":"{{0,0}}"
                    }}
                }}
            }},
            "fluid":
            {{
                "body":
                {{
                    "Sphere0":
                    {{
                        "markers":["Sphere0"],
                        "materials":
                        {{
                            "names":["Sph0"]
                        }}
                    }}{Body_Block_Nsphere}
                }}
            }}
        }},
        "PostProcess":
        {{
            "Exports":
            {{
                "fields":["velocity","pressure","pid","displacement"]
            }},
            "Measures":
            {{
                "Quantities":
                {{
                    "names":"all"
                }}
            }}
        }}
    }}"""
        Material_Block_Nsphere=""",
            "Sph"""+str(0)+"""":{
                "physics":"body",
                "rho":1e-1
            }"""
        Material_Block_Nsphere=Material_Block_Nsphere+""",
            "Sph"""+str(1)+"""":{
                "physics":"body",
                "rho":1e-1
            }"""
        Body_Block_Nsphere=""",
                    "Sphere"""+str(1)+"""":
                    {
                        "markers":["Sphere"""+str(1)+""""],
                        "materials":
                        {
                            "names":["Sph"""+str(1)+""""]
                        },
                        "articulation":
                        {
                            "body":"Sphere"""+str(1-1)+"""",
                            "translational-velocity":" """+str(Relative_Vel[0])+""""
                        }
                    }"""
        for i in range(2,N):
            Material_Block_Nsphere=Material_Block_Nsphere+""",
            "Sph"""+str(i)+"""":{
                "physics":"body",
                "rho":1e-1
            }"""
            Body_Block_Nsphere=Body_Block_Nsphere+""",
                    "Sphere"""+str(i)+"""":
                    {
                        "markers":["Sphere"""+str(i)+""""],
                        "materials":
                        {
                            "names":["Sph"""+str(i)+""""]
                        },
                        "articulation":
                        {
                            "body":"Sphere"""+str(i-1)+"""",
                            "translational-velocity":" """+str(Relative_Vel[i-1])+""""
                        }
                    }"""
        context = {"N":N,
        "Material_Block_Nsphere":Material_Block_Nsphere,
        "Body_Block_Nsphere":Body_Block_Nsphere,
        "RelativeVelocity":RelativeVelocity}
        with  open(os.path.join(path,title),'w') as myfile:
            myfile.write(template.format(**context))
    else:
        print('Dimension not coded for the moment')

#-------------------GEO FILE--------------#
# it repeats the same pattern for the creation of a new sphere, adding i*100 to get the new indices,
# while the indices of the bounding box are N*100
# the spheres are numbered from the left to the right, from 1 to N
# the flags are "BoxWalls" and "Fluid"
# "Spherei", for i from 0 to N-1 for the surface tag
# "Sphi", for i from 0 to N-1 for the volume tag
def write_geo(N,state,path,Dim,L,Radius):
    eps = 4
    N_true = 0
    N_false = 0
    arms_length = []
    s = 0
    for j in range(0, N-1):
        if state[j] :
            N_true += 1
            arms_length.append(L)
        else :
            N_false += 1
            arms_length.append(L-eps)
    if Dim==2:
        title=str(N)+'_sphere.geo'
        template="""RSphere ="""+str(Radius)+""";
    h=3;
    lcSphere = h/10;//0.3;
    lcDom = h;//5.;
    Center0 = """+str(-N_true/2*L - N_false/2*(L-eps))+""";
    Centery = 0;
    Point(1) = {Center0,Centery,0,lcSphere};
    Point(2) = {Center0+RSphere,Centery,0,lcSphere};
    Point(3) = {Center0-RSphere,Centery,0,lcSphere};
    Circle(1) = {2, 1, 3};
    Circle(2) = {3, 1, 2};
    Line Loop(4) = {1, 2};
    Plane Surface(5) = {4};
    Physical Curve("Sphere0") = {1, 2};
    Physical Surface("Sph0") = {5};"""
        x_position = -N_true/2*L - N_false/2*(L-eps)
        for i in range(1,N) :
            s = 0
            for j in range(0, i):
                s += arms_length[j]
            template=template+"""
    Center"""+str(i)+""" ="""+str(x_position + s)+""";
    Centery = 0;
    Point("""+str(1+i*100)+""") = {Center"""+str(i)+""",Centery,0,lcSphere};
    Point("""+str(2+i*100)+""") = {Center"""+str(i)+"""+RSphere,Centery,0,lcSphere};
    Point("""+str(3+i*100)+""") = {Center"""+str(i)+"""-RSphere,Centery,0,lcSphere};
    Circle("""+str(1+i*100)+""") = {"""+str(2+i*100)+""","""+str(1+i*100)+""","""+str(3+i*100)+"""};
    Circle("""+str(2+i*100)+""") = {"""+str(3+i*100)+""","""+str(1+i*100)+""","""+str(2+i*100)+"""};
    Line Loop("""+str(4+i*100)+""") = {"""+str(1+i*100)+""","""+str(2+i*100)+"""};
    Plane Surface("""+str(5+i*100)+""") = {"""+str(4+i*100)+"""};
    Physical Curve("Sphere"""+str(i)+"""") = {"""+str(1+i*100)+""", """+str(2+i*100)+"""};
    Physical Surface("Sph"""+str(i)+"""") = {"""+str(5+i*100)+"""};"""
            #x_position += arms_length[i-1]
        
        template=template+"""
    HalfSide = """+str(N*3/4*L)+""";
    HalfHeight ="""+str(2*L)+""";
    Point("""+str(1+N*100)+""") = {-HalfSide, -HalfHeight, 0,lcDom};
    Point("""+str(2+N*100)+""") = {HalfSide, -HalfHeight, 0,lcDom};
    Point("""+str(3+N*100)+""") = {HalfSide, HalfHeight, 0,lcDom};
    Point("""+str(4+N*100)+""") = {-HalfSide, HalfHeight, 0,lcDom};
    Line("""+str(13+N*100)+""") = {"""+str(1+N*100)+""", """+str(2+N*100)+"""};
    Line("""+str(14+N*100)+""") = {"""+str(2+N*100)+""", """+str(3+N*100)+"""};
    Line("""+str(15+N*100)+""") = {"""+str(3+N*100)+""", """+str(4+N*100)+"""};
    Line("""+str(16+N*100)+""") = {"""+str(4+N*100)+""", """+str(1+N*100)+"""};
    Line Loop("""+str(28+N*100)+""") = {"""+str(13+N*100)+""", """+str(14+N*100)+""", """+str(15+N*100)+""", """+str(16+N*100)+"""};
    Plane Surface("""+str(29+N*100)+""") = {"""
        for i in range(0,N):
            template=template+str(4+i*100)+""","""
        template=template+str(28+N*100)+"""};
    Physical Curve("BoxWalls") = {"""+str(13+N*100)+""", """+str(14+N*100)+""", """+str(15+N*100)+""", """+str(16+N*100)+"""};
    Physical Surface("Fluid") = {"""+str(29+N*100)+"""};
        """
        with  open(os.path.join(path,title),'w') as myfile:
            myfile.write(template)
    else:
        print('Dimension not coded')


def write_preconditioner(N,path,Dim):
    if Dim==2:
        title=str(N)+"_preconditioner.cfg"
        template="""[fluid]
    preconditioner.attach-pmm=1
    #pc-view=1
    [fluid]
    ksp-type=fgmres#gcr#fgmres
    fgmres-restart=100
    # fieldsplit
    pc-type=fieldsplit
    fieldsplit-type=schur #additive, multiplicative, schur
    fieldsplit-schur-fact-type=upper#full#upper#full #diag, lower, upper, full
    fieldsplit-schur-precondition=self
    fieldsplit-fields=0->(0"""
        for i in range (2,2*N+N+1):
            template=template+""","""+str(i)
        template=template+"""),1->(1)
    # block velocity
    [fluid.fieldsplit-0]
    ksp-type=preonly#gmres#preonly
    ksp-maxit=2#5#
    pc-type=fieldsplit
    fieldsplit-fields=0->(0),1->(2"""
        for i in range (3,2*N+N+1):
            template=template+""","""+str(i)
        template=template+""")
    fieldsplit-type=multiplicative #additive
    [fluid.fieldsplit-0.fieldsplit-0]
    ksp-type=gmres#preonly
    ksp-rtol=1e-3
    pc-type=gamg#gasm#lu#bjacobi#lu#gasm#boomeramg#gamg#lu
    #pc-gamg-nsmooths=2
    ksp-maxit=5#30#5#30
    #pc-factor-mat-solver-package-type=mumps
    #pc-factor-mumps.icntl-14=60
    [fluid.fieldsplit-0.fieldsplit-1]
    ksp-type=preonly
    pc-type=lu
    pc-factor-mat-solver-package-type=mumps
    [fluid.fieldsplit-1]
    ksp-type=gmres#preonly#gmres
    ksp-maxit=1#15
    ksp-rtol=1e-3
    pc-type=pmm
    pmm.pc-type=gamg#lu
    pmm.ksp-type=preonly#gmres
    #pmm.ksp-view=1
    #pmm.ksp-monitor=1
    #pc-factor-mat-solver-package-type=mumps
    #pc-factor-mumps.icntl-14=60
    """
        with  open(os.path.join(path,title),'w') as myfile:
            myfile.write(template)
    else:
        print('Dimension not coded')

